'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useRowApi = exports.useColApi = exports.layoutApi = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var flex = { display: 'flex' };
var row = flex;
var col = _extends({}, flex, { flexDirection: 'column' });

var layoutApi = exports.layoutApi = {
  row: row,
  col: col,
  rowx: { justifyContent: 'center' },
  rowy: { alignItems: 'center' },
  colx: { alignItems: 'center' },
  coly: { justifyContent: 'center' },
  rowxEnd: {
    justifyContent: 'flex-end'
  },
  colxEnd: {
    alignItems: 'flex-end'
  },
  rowyEnd: {
    alignItems: 'flex-end'
  },
  colyEnd: {
    justifyContent: 'flex-end'
  },
  reverse: {
    flexDirection: 'row-reverse'
  },
  columnReverse: {
    flexDirection: 'column-reverse'
  },
  wrap: {
    flexWrap: 'wrap'
  },
  wrapReverse: {
    flexWrap: 'wrap-reverse'
  },
  spaceBetween: {
    justifyContent: 'space-between'
  },
  spaceAround: {
    justifyContent: 'space-around'
  }
};

var axisConfig = {
  row: {
    x: {
      true: 'rowx',
      end: 'rowxEnd'
    },
    y: {
      true: 'rowy',
      end: 'rowyEnd'
    }
  },
  col: {
    x: {
      true: 'colx',
      end: 'colxEnd'
    },
    y: {
      true: 'coly',
      end: 'colyEnd'
    }
  }
};

var useColApi = exports.useColApi = useLayoutApi('col');
var useRowApi = exports.useRowApi = useLayoutApi('row');

/**
 * 1. When applying alignItems or JustifyContent properties we will give
 * preference to the spaceProp by placing it last.
 */
function propAdaptors(type) {
  return [function (_ref) {
    var x = _ref.x;
    return axisConfig[type].x[x];
  }, function (_ref2) {
    var y = _ref2.y;
    return axisConfig[type].y[y];
  }, function (_ref3) {
    var wrap = _ref3.wrap;

    if (wrap === true) {
      return 'wrap';
    } else if (wrap === 'reverse') {
      return 'wrapReverse';
    } else {
      return false;
    }
  }, function (_ref4) {
    var reverse = _ref4.reverse;

    if (!reverse) {
      return false;
    } else if (type === 'row') {
      return 'reverse';
    } else {
      return 'columnReverse';
    }
  }, function (_ref5) {
    var space = _ref5.space;
    return space ? 'space' + (0, _utils.capitalize)(space) : false;
  } /* [1] */
  ];
}

function useLayoutApi(type) {
  var adaptors = propAdaptors(type);
  return function (Style) {
    var LayoutApi = function LayoutApi(props) {
      var adaptedProps = _defineProperty({}, type, true);
      adaptors.forEach(function (adaptor) {
        var value = adaptor(props);
        if (value) {
          adaptedProps[value] = true;
        }
      });

      return _react2.default.createElement(Style, _extends({}, (0, _utils.pluck)(['x', 'y', 'reverse', 'wrap', 'space'], props), adaptedProps));
    };

    return LayoutApi;
  };
}